<?php
/*
 * This file is part of the Manualranking Product plugin
 *
 * Copyright (C) 2016 LOCKON CO.,LTD. All Rights Reserved.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\Manualranking\Controller;

use Eccube\Application;
use Eccube\Controller\AbstractController;
use Plugin\Manualranking\Entity\ManualrankingProduct;
use Symfony\Component\Form\Form;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Class ManualrankingController.
 */
class ManualrankingController extends AbstractController
{
    /**
     * 手動売上ランキング一覧.
     *
     * @param Application $app
     * @param Request     $request
     *
     * @return Response
     */
    public function index(Application $app, Request $request)
    {
        $pagination = $app['eccube.plugin.manualranking.repository.manualranking_product']->getManualrankingList();

        return $app->render('Manualranking/Resource/template/admin/index.twig', array(
            'pagination' => $pagination,
            'total_item_count' => count($pagination),
        ));
    }

    /**
     * Create & Edit.
     *
     * @param Application $app
     * @param Request     $request
     * @param int         $id
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function edit(Application $app, Request $request, $id = null)
    {
        /* @var ManualrankingProduct $Manualranking */
        $Manualranking = null;
        $Product = null;
        if (!is_null($id)) {
            // IDから手動売上ランキング情報を取得する
            $Manualranking = $app['eccube.plugin.manualranking.repository.manualranking_product']->find($id);

            if (!$Manualranking) {
                $app->addError('admin.manualranking.not_found', 'admin');
                log_info('The manualranking product is not found.', array('Manualranking id' => $id));

                return $app->redirect($app->url('admin_manualranking_list'));
            }

            $Product = $Manualranking->getProduct();
        }

        // formの作成
        /* @var Form $form */
        $form = $app['form.factory']
            ->createBuilder('admin_manualranking', $Manualranking)
            ->getForm();

        $form->handleRequest($request);
        $data = $form->getData();
        if ($form->isSubmitted() && $form->isValid()) {
            $service = $app['eccube.plugin.manualranking.service.manualranking'];
            if (is_null($data['id'])) {
                if ($status = $service->createManualranking($data)) {
                    $app->addSuccess('admin.plugin.manualranking.register.success', 'admin');
                    log_info('Add the new manualranking product success.', array('Product id' => $data['Product']->getId()));
                }
            } else {
                if ($status = $service->updateManualranking($data)) {
                    $app->addSuccess('admin.plugin.manualranking.update.success', 'admin');
                    log_info('Update the manualranking product success.', array('Manualranking id' => $Manualranking->getId(), 'Product id' => $data['Product']->getId()));
                }
            }

            if (!$status) {
                $app->addError('admin.manualranking.not_found', 'admin');
                log_info('Failed the manualranking product updating.', array('Product id' => $data['Product']->getId()));
            }

            return $app->redirect($app->url('admin_manualranking_list'));
        }

        if (!empty($data['Product'])) {
            $Product = $data['Product'];
        }

        $arrProductIdByManualranking = $app['eccube.plugin.manualranking.repository.manualranking_product']->getManualrankingProductIdAll();

        return $this->registerView(
            $app,
            array(
                'form' => $form->createView(),
                'manualranking_products' => json_encode($arrProductIdByManualranking),
                'Product' => $Product,
            )
        );
    }

    /**
     * 手動売上ランキングの削除.
     *
     * @param Application $app
     * @param Request     $request
     * @param int         $id
     *
     * @throws BadRequestHttpException
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function delete(Application $app, Request $request, $id)
    {
        // Valid token
        $this->isTokenValid($app);

        // Check request
        if (!'POST' === $request->getMethod()) {
            log_error('Delete with bad method!');
            throw new BadRequestHttpException();
        }

        // Id valid
        if (!$id) {
            $app->addError('admin.manualranking.manualranking_id.not_exists', 'admin');

            return $app->redirect($app->url('admin_manualranking_list'));
        }

        $service = $app['eccube.plugin.manualranking.service.manualranking'];

        // 手動売上ランキング情報を削除する
        if ($service->deleteManualranking($id)) {
            log_info('The manualranking product delete success!', array('Manualranking id' => $id));
            $app->addSuccess('admin.plugin.manualranking.delete.success', 'admin');
        } else {
            $app->addError('admin.manualranking.not_found', 'admin');
            log_info('The manualranking product is not found.', array('Manualranking id' => $id));
        }

        return $app->redirect($app->url('admin_manualranking_list'));
    }

    /**
     * Move rank with ajax.
     *
     * @param Application $app
     * @param Request     $request
     *
     * @return bool
     */
    public function moveRank(Application $app, Request $request)
    {
        if ($request->isXmlHttpRequest()) {
            $arrRank = $request->request->all();
            $arrRankMoved = $app['eccube.plugin.manualranking.repository.manualranking_product']->moveManualrankingRank($arrRank);
            log_info('Manualranking move rank', $arrRankMoved);
        }

        return true;
    }

    /**
     * 編集画面用のrender.
     *
     * @param Application $app
     * @param array       $parameters
     *
     * @return Response
     */
    protected function registerView($app, $parameters = array())
    {
        // 商品検索フォーム
        $searchProductModalForm = $app['form.factory']->createBuilder('admin_search_product')->getForm();
        $viewParameters = array(
            'searchProductModalForm' => $searchProductModalForm->createView(),
        );
        $viewParameters += $parameters;

        return $app->render('Manualranking/Resource/template/admin/regist.twig', $viewParameters);
    }
}
